package com.example.PhidgetDCMotorExample;

import android.app.Activity;
import android.content.Context;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.widget.CheckBox;
import android.widget.LinearLayout;
import android.widget.SeekBar;
import android.widget.TextView;
import android.widget.Toast;

import com.phidget22.*;

import java.text.DecimalFormat;

public class DCMotorExample extends Activity {

	DCMotor ch;
	SeekBar accelerationBar;
    SeekBar targetVelocityBar;

	Toast errToast;

	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);

        //Hide device information and settings until one is attached
		LinearLayout settingsAndData = (LinearLayout) findViewById(R.id.settingsAndData);
		settingsAndData.setVisibility(LinearLayout.GONE);

		//set acceleration seek bar functionality
		accelerationBar = (SeekBar) findViewById(R.id.accelerationBar);
		accelerationBar.setOnSeekBarChangeListener(new accelerationChangeListener());

		//set target velocity seek bar functionality
		targetVelocityBar = (SeekBar) findViewById(R.id.targetVelocityBar);
		targetVelocityBar.setOnSeekBarChangeListener(new targetVelocityChangeListener());

        try
        {
        	ch = new DCMotor();

        	//Allow direct USB connection of Phidgets
			if(getPackageManager().hasSystemFeature(PackageManager.FEATURE_USB_HOST))
                com.phidget22.usb.Manager.Initialize(this);

			//Enable server discovery to list remote Phidgets
			this.getSystemService(Context.NSD_SERVICE);
			Net.enableServerDiscovery(ServerType.DEVICE_REMOTE);

			ch.addAttachListener(new AttachListener() {
				public void onAttach(final AttachEvent attachEvent) {
				    AttachEventHandler handler = new AttachEventHandler(ch);
					synchronized(handler)
					{
						runOnUiThread(handler);
						try {
							handler.wait();
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
					}
				}
			});

			ch.addDetachListener(new DetachListener() {
				public void onDetach(final DetachEvent detachEvent) {
                    DetachEventHandler handler = new DetachEventHandler(ch);
					synchronized(handler)
					{
						runOnUiThread(handler);
						try {
							handler.wait();
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
					}
				}
			});

			ch.addErrorListener(new ErrorListener() {
				public void onError(final ErrorEvent errorEvent) {
					ErrorEventHandler handler = new ErrorEventHandler(ch, errorEvent);
					runOnUiThread(handler);
				}
			});

			ch.addVelocityUpdateListener(new DCMotorVelocityUpdateListener() {
				public void onVelocityUpdate(DCMotorVelocityUpdateEvent velocityUpdateEvent) {
                    DCMotorVelocityUpdateEventHandler handler = new DCMotorVelocityUpdateEventHandler(ch, velocityUpdateEvent);
                    runOnUiThread(handler);
                }
			});

			ch.open();
        } catch (PhidgetException pe) {
	        pe.printStackTrace();
		}

    }

	private class accelerationChangeListener implements SeekBar.OnSeekBarChangeListener {
		public void onProgressChanged(SeekBar seekBar, int progress,
									  boolean fromUser) {
			try {
				TextView accelerationTxt = (TextView) findViewById(R.id.accelerationTxt);
				DecimalFormat numberFormat = new DecimalFormat("#.##");
				double acceleration = ((double) progress / seekBar.getMax()) *
						(ch.getMaxAcceleration() - ch.getMinAcceleration()) + ch.getMinAcceleration();

				accelerationTxt.setText(numberFormat.format(acceleration));
				ch.setAcceleration(acceleration);
			} catch (PhidgetException e) {
				e.printStackTrace();
			}
		}

		public void onStartTrackingTouch(SeekBar seekBar) {}

		public void onStopTrackingTouch(SeekBar seekBar) {}
	}

    private class targetVelocityChangeListener implements SeekBar.OnSeekBarChangeListener {
        public void onProgressChanged(SeekBar seekBar, int progress,
                                      boolean fromUser) {
            try {
                TextView targetVelocityTxt = (TextView) findViewById(R.id.targetVelocityTxt);
                double targetVelocity = ((double)progress - seekBar.getMax()/2) / (targetVelocityBar.getMax()/2);
				targetVelocityTxt.setText(String.valueOf(targetVelocity));
                ch.setTargetVelocity(targetVelocity);
            } catch (PhidgetException e) {
                e.printStackTrace();
            }
        }

        public void onStartTrackingTouch(SeekBar seekBar) {}

        public void onStopTrackingTouch(SeekBar seekBar) {}
    }

    class AttachEventHandler implements Runnable { 
    	Phidget ch;

		public AttachEventHandler(Phidget ch) {
			this.ch = ch;
		}

		public void run() {
			LinearLayout settingsAndData = (LinearLayout) findViewById(R.id.settingsAndData);
			settingsAndData.setVisibility(LinearLayout.VISIBLE);

			TextView attachedTxt = (TextView) findViewById(R.id.attachedTxt);

			attachedTxt.setText("Attached");
			try {
				TextView nameTxt = (TextView) findViewById(R.id.nameTxt);
				TextView serialTxt = (TextView) findViewById(R.id.serialTxt);
				TextView versionTxt = (TextView) findViewById(R.id.versionTxt);
				TextView channelTxt = (TextView) findViewById(R.id.channelTxt);
				TextView hubPortTxt = (TextView) findViewById(R.id.hubPortTxt);
				TextView labelTxt = (TextView) findViewById(R.id.labelTxt);

				nameTxt.setText(ch.getDeviceName());
				serialTxt.setText(Integer.toString(ch.getDeviceSerialNumber()));
				versionTxt.setText(Integer.toString(ch.getDeviceVersion()));
				channelTxt.setText(Integer.toString(ch.getChannel()));
				hubPortTxt.setText(Integer.toString(ch.getHubPort()));
				labelTxt.setText(ch.getDeviceLabel());

				TextView accelerationTxt = (TextView) findViewById(R.id.accelerationTxt);
				accelerationTxt.setText(String.valueOf(((DCMotor)ch).getAcceleration()));

				SeekBar accelerationBar = (SeekBar) findViewById(R.id.accelerationBar);
				accelerationBar.setProgress((int)((((DCMotor)ch).getAcceleration() - ((DCMotor)ch).getMinAcceleration())
						/(((DCMotor)ch).getMaxAcceleration() - ((DCMotor)ch).getMinAcceleration()) * accelerationBar.getMax()));

                TextView targetVelocityTxt = (TextView) findViewById(R.id.targetVelocityTxt);
				targetVelocityTxt.setText(String.valueOf(((DCMotor)ch).getVelocity()));

                SeekBar targetVelocityBar = (SeekBar) findViewById(R.id.targetVelocityBar);
				targetVelocityBar.setProgress((int)((((DCMotor)ch).getVelocity() * targetVelocityBar.getMax()/2) + targetVelocityBar.getMax()/2));

			} catch (PhidgetException e) {
				e.printStackTrace();
			}

			//notify that we're done
			synchronized(this)
			{
				this.notify();
			}
		}
    }
    
    class DetachEventHandler implements Runnable {
    	Phidget ch;
    	
    	public DetachEventHandler(Phidget ch) {
    		this.ch = ch;
    	}
    	
		public void run() {
			LinearLayout settingsAndData = (LinearLayout) findViewById(R.id.settingsAndData);

			settingsAndData.setVisibility(LinearLayout.GONE);

			TextView attachedTxt = (TextView) findViewById(R.id.attachedTxt);
			attachedTxt.setText("Detached");

			TextView nameTxt = (TextView) findViewById(R.id.nameTxt);
			TextView serialTxt = (TextView) findViewById(R.id.serialTxt);
			TextView versionTxt = (TextView) findViewById(R.id.versionTxt);
			TextView channelTxt = (TextView) findViewById(R.id.channelTxt);
			TextView hubPortTxt = (TextView) findViewById(R.id.hubPortTxt);
			TextView labelTxt = (TextView) findViewById(R.id.labelTxt);

			nameTxt.setText(R.string.unknown_val);
			serialTxt.setText(R.string.unknown_val);
			versionTxt.setText(R.string.unknown_val);
			channelTxt.setText(R.string.unknown_val);
			hubPortTxt.setText(R.string.unknown_val);
			labelTxt.setText(R.string.unknown_val);

			((TextView)findViewById(R.id.velocityTxt)).setText("");

			//notify that we're done
			synchronized(this)
			{
				this.notify();
			}
		}
    }

	class ErrorEventHandler implements Runnable {
		Phidget ch;
		ErrorEvent errorEvent;

		public ErrorEventHandler(Phidget ch, ErrorEvent errorEvent) {
			this.ch = ch;
			this.errorEvent = errorEvent;
		}

		public void run() {
			 if (errToast == null)
				 errToast = Toast.makeText(getApplicationContext(), errorEvent.getDescription(), Toast.LENGTH_SHORT);

			 //replace the previous toast message if a new error occurs
			 errToast.setText(errorEvent.getDescription());
			 errToast.show();
        }
	}

	class DCMotorVelocityUpdateEventHandler implements Runnable {
		Phidget ch;
		DCMotorVelocityUpdateEvent velocityUpdateEvent;

		public DCMotorVelocityUpdateEventHandler(Phidget ch, DCMotorVelocityUpdateEvent velocityUpdateEvent) {
			this.ch = ch;
			this.velocityUpdateEvent = velocityUpdateEvent;
		}

		public void run() {
			TextView velocityTxt = (TextView)findViewById(R.id.velocityTxt);
			velocityTxt.setText(String.valueOf(velocityUpdateEvent.getVelocity()));
		}
	}

    @Override
    protected void onDestroy() {
    	super.onDestroy();
    	try {
			ch.close();

		} catch (PhidgetException e) {
			e.printStackTrace();
		}

		//Disable USB connection to Phidgets
    	if(getPackageManager().hasSystemFeature(PackageManager.FEATURE_USB_HOST))
            com.phidget22.usb.Manager.Uninitialize();
    }

}

